@@ -1,26 +1,23 @@
 """User configuration of moe.
 
-To avoid namespace confusion when using a variable named config, typical usage of this
-module should just import the Config class directly::
+Each instance of Moe should only create a single `Config` object. This object will
+contain all user settings and other related information and is set upon initializing
+the configuration for the first time i.e. calling `Config()`. Once initialized, the
+config can be accessed through import and should be treated as a constant::
 
-    from moe.config import Config
-    config = Config()
-
-A configuration shuold only be instantiated once per instance of Moe. Plugins should
-pass along that instance through their hooks.
+    from moe import config
+    print(config.CONFIG.settings.library_path)
 """
 
-import functools
 import importlib
 import importlib.util
 import logging
 import os
 import re
-import sys
 from contextlib import suppress
 from pathlib import Path
 from types import ModuleType
-from typing import Any, List, NamedTuple, Optional, Type, Union
+from typing import NamedTuple, Optional, Union, cast
 
 import dynaconf
 import pluggy
@@ -31,19 +28,18 @@ import sqlalchemy.orm
 import alembic.command
 import alembic.config
 import moe
-import moe.config
-from moe.library.lib_item import LibItem, PathType
 
-session_factory = sqlalchemy.orm.sessionmaker()
+session_factory = sqlalchemy.orm.sessionmaker(autoflush=False)
 MoeSession = sqlalchemy.orm.scoped_session(session_factory)
 
-__all__ = ["Config", "ExtraPlugin"]
+__all__ = ["CONFIG", "Config", "ConfigValidationError", "ExtraPlugin"]
 
 log = logging.getLogger("moe.config")
 
 DEFAULT_PLUGINS = (
     "add",
     "cli",
+    "duplicate",
     "edit",
     "import",
     "info",
@@ -53,6 +49,19 @@ DEFAULT_PLUGINS = (
     "remove",
     "write",
 )
+CORE_PLUGINS = {
+    "config": "moe.config",
+    "album": "moe.library.album",
+    "extra": "moe.library.extra",
+    "track": "moe.library.track",
+    "lib_item": "moe.library.lib_item",
+}  # {name: module} of plugins that cannot be overwritten by the config
+
+CONFIG = cast("Config", None)
+
+
+class ConfigValidationError(Exception):
+    """Error in the user's configuration."""
 
 
 class Hooks:
@@ -79,56 +88,26 @@ class Hooks:
 
     @staticmethod
     @moe.hookspec
-    def add_hooks(plugin_manager: pluggy.manager.PluginManager):
+    def add_hooks(pm: pluggy.manager.PluginManager):
         """Add hookspecs to be registered to Moe.
 
         Args:
-            plugin_manager: PluginManager that registers the hookspec.
+            pm: PluginManager that registers the hookspec.
 
         Example:
             Inside your hook implementation::
 
                 from moe.plugins.add import Hooks
-                plugin_manager.add_hookspecs(Hooks)
+                pm.add_hookspecs(Hooks)
         """
 
     @staticmethod
     @moe.hookspec
-    def edit_new_items(config: "Config", items: List[LibItem]):
-        """Edit any new or changed items prior to them being added to the library.
-
-        Args:
-            config: Moe config.
-            items: Any new or changed items in the current session. The items and
-                their changes have not yet been committed to the library.
-
-        See Also:
-            The :meth:`process_new_items` hook if you wish to process any items after
-            any final edits have been made and they have been successfully added to
-            the library.
-        """
-
-    @staticmethod
-    @moe.hookspec
-    def process_new_items(config: "Config", items: List[LibItem]):
-        """Process any new or changed items after they have been added to the library.
-
-        Args:
-            config: Moe config.
-            items: Any new or changed items that have been successfully added to the
-                library during the current session.
-
-        See Also:
-            The :meth:`edit_new_items` hook if you wish to edit the items.
-        """
-
-    @staticmethod
-    @moe.hookspec
-    def plugin_registration(config: "Config"):
+    def plugin_registration():
         """Allows actions after the initial plugin registration.
 
         In order for a module to implement and register plugin hooks, it must be
-        registered as a separate plugin with the ``plugin_manager``. A plugin can be
+        registered as a separate plugin with the ``pm``. A plugin can be
         either just a module, or a full package.
 
         If a plugin is a package, only it's ``__init__.py`` will be initially
@@ -144,9 +123,9 @@ class Hooks:
         sub-module::
 
             @moe.hookimpl
-            def plugin_registration(config):
-                if config.plugin_manager.has_plugin("cli"):
-                    config.plugin_manager.register(edit_cli, "edit_cli")
+            def plugin_registration():
+                if config.CONFIG.pm.has_plugin("cli"):
+                    config.CONFIG.pm.register(edit_cli, "edit_cli")
 
         This hook can also be used as a way of checking for plugin dependencies by
         inspecting the enabled plugins in the configuration.
@@ -155,27 +134,57 @@ class Hooks:
         un-register itself and log a warning if the cli plugin is not enabled::
 
             @moe.hookimpl
-            def plugin_registration(config):
-                if not config.plugin_manager.has_plugin("cli"):
-                    config.plugin_manager.set_blocked("list")
+            def plugin_registration():
+                if not config.CONFIG.pm.has_plugin("cli"):
+                    config.CONFIG.pm.set_blocked("list")
                     log.warning("You can't list stuff without a cli!")
 
         See Also:
-            The ``PluginManager`` api documentation:
-            https://pluggy.readthedocs.io/en/latest/api_reference.html
+            `pluggy.PluginManager documentation <https://pluggy.readthedocs.io/en/latest/api_reference.html>`_
+        """  # noqa: E501
+
+    @staticmethod
+    @moe.hookspec
+    def register_sa_event_listeners(session: sqlalchemy.orm.Session):
+        """Registers new sqlalchemy event listeners.
 
         Args:
-            config: Moe config.
-        """
+            session: Session to attach the listener to.
+
+        Important:
+            This hooks is for Moe internal use only and should not be used by plugins.
+
+        Example:
+            In your hook implementation::
+
+                sqlalchemy.event.listen(
+                    session,
+                    "before_flush",
+                    _my_func,
+                )
+
+            Then you can define ``_my_func`` as such::
+
+                def _my_func(
+                    session: sqlalchemy.orm.Session,
+                    flush_context: sqlalchemy.orm.UOWTransaction,
+                    instances: Optional[Any],
+                ):
+                    print("we made it")
+
+        See Also:
+            `SQLAlchemy ORM even documentation <https://docs.sqlalchemy.org/en/14/orm/events.html#orm-events>`_
+        """  # noqa: E501
 
 
 @moe.hookimpl
 def add_config_validator(settings: dynaconf.base.LazySettings):
     """Validate move plugin configuration settings."""
-    settings.validators.register(
-        dynaconf.Validator("DEFAULT_PLUGINS", default=list(DEFAULT_PLUGINS))
-    )
-    settings.validators.register(dynaconf.Validator("LIBRARY_PATH", default="~/Music"))
+    validators = [
+        dynaconf.Validator("DEFAULT_PLUGINS", default=list(DEFAULT_PLUGINS)),
+        dynaconf.Validator("LIBRARY_PATH", default="~/Music"),
+    ]
+    settings.validators.register(*validators)
 
 
 class ExtraPlugin(NamedTuple):
@@ -186,7 +195,7 @@ class ExtraPlugin(NamedTuple):
         name: Name to register the plugin under.
     """
 
-    plugin: Union[Type, ModuleType]
+    plugin: Union[type, ModuleType]
     name: str
 
 
@@ -197,24 +206,15 @@ class Config:
         config_dir (Path): Filesystem path of the configuration directory.
         config_file (Path): Filesystem path of the configuration settings file.
         engine (sa.engine.base.Engine): Database engine in use.
-        plugin_manager (pluggy.manager.PluginManager): Manages plugin logic.
+        pm (pluggy.manager.PluginManager): Plugin manager that handles plugin logic.
         settings (dynaconf.base.LazySettings): User configuration settings.
-
-    Example:
-        In your plugin, to access the library_path setting (assuming a Config object
-        named config)::
-
-            config.settings.library_path
-
-        See the dynaconf documentation for more info on reading settings variables.
-        https://www.dynaconf.com/#reading-settings-variables
     """
 
     def __init__(
         self,
         config_dir: Path = Path.home() / ".config" / "moe",  # noqa: B008
         settings_filename: str = "config.toml",
-        extra_plugins: Optional[List[ExtraPlugin]] = None,
+        extra_plugins: Optional[list[ExtraPlugin]] = None,
         engine: Optional[sqlalchemy.engine.base.Engine] = None,
         init_db=True,
     ):
@@ -231,10 +231,14 @@ class Config:
                 in the ``config_dir``.
             init_db: Whether or not to initialize the database.
         """
+        global CONFIG
+        CONFIG = self
+
         try:
             self.config_dir = Path(os.environ["MOE_CONFIG_DIR"])
         except KeyError:
             self.config_dir = config_dir
+
         self.config_dir.mkdir(parents=True, exist_ok=True)
 
         self.config_file = self.config_dir / settings_filename
@@ -254,6 +258,8 @@ class Config:
             create_tables: Whether or not to create and update the db tables.
                 If doing db migrations manually, e.g. in alembic, this should be False.
         """
+        log.debug(f"Initializing database. [{create_tables=!r}]")
+
         db_path = self.config_dir / "library.db"
 
         if not self.engine:
@@ -272,28 +278,12 @@ class Config:
                 alembic_cfg.attributes["connection"] = connection
                 alembic.command.upgrade(alembic_cfg, "head")
 
-        # initialize sqlalchemy event listeners
-        session = MoeSession()
-        sqlalchemy.event.listen(
-            session,
-            "before_flush",
-            functools.partial(_edit_new_items, config=self),
-        )
-        sqlalchemy.event.listen(
-            session,
-            "after_flush",
-            functools.partial(_process_new_items, config=self),
-        )
+        self.pm.hook.register_sa_event_listeners(config=self, session=MoeSession())
 
         # create regular expression function for sqlite queries
         @sqlalchemy.event.listens_for(self.engine, "begin")
         def sqlite_engine_connect(conn):
-            if (sys.version_info.major, sys.version_info.minor) < (3, 8):
-                conn.connection.create_function("regexp", 2, _regexp)
-            else:
-                conn.connection.create_function(
-                    "regexp", 2, _regexp, deterministic=True
-                )
+            conn.connection.create_function("regexp", 2, _regexp, deterministic=True)
 
         def _regexp(pattern: str, col_value) -> bool:
             """Use the python re module for sqlite regular expression functionality.
@@ -308,11 +298,20 @@ class Config:
             """
             return re.search(pattern, str(col_value), re.IGNORECASE) is not None
 
+        log.debug(f"Initialized database. [engine={self.engine!r}]")
+
     def _read_config(self):
         """Reads the user configuration settings.
 
         Searches for a configuration file at `config_dir / "config.toml"`.
+
+        Raises:
+            ConfigValidationError: Unable to parse the configuration file.
         """
+        from moe.library.lib_item import PathType
+
+        log.debug(f"Reading configuration file. [config_file={self.config_file}]")
+
         self.config_file.touch(exist_ok=True)
 
         self.settings = dynaconf.Dynaconf(
@@ -321,21 +320,45 @@ class Config:
         )
 
         self._setup_plugins()
-        self.plugin_manager.hook.add_config_validator(settings=self.settings)
+        self.pm.hook.add_config_validator(settings=self.settings)
+        self._validate_settings()
 
-        self.settings.validators.validate()
         PathType.library_path = Path(self.settings.library_path)
 
-    def _setup_plugins(self):
-        """Setup plugin_manager and hook logic."""
-        self.plugin_manager = pluggy.PluginManager("moe")
+    def _validate_settings(self):
+        """Validates the given user configuration.
+
+        Raises:
+            ConfigValidationError: Error validating the configuration.
+        """
+        log.debug("Validating configuration settings.")
+
+        try:
+            self.settings.validators.validate()
+        except dynaconf.validator.ValidationError as err:
+            raise ConfigValidationError(err) from err
+
+    def _setup_plugins(self, core_plugins: dict[str, str] = CORE_PLUGINS):
+        """Setup pm and hook logic.
+
+        Args:
+            core_plugins: Optional mapping of core plugin modules to names.
+                These plugins cannot be overwritten by the user configuration.
+        """
+        log.debug("Setting up plugins.")
+
+        self.pm = pluggy.PluginManager("moe")
+
+        # register core modules that are not considered plugins
+        for plugin_name, module in core_plugins.items():
+            self.pm.register(importlib.import_module(module), plugin_name)
 
         # need to validate `config` specific settings separately so we have access to
         # the 'default_plugins' setting
-        self.plugin_manager.register(moe.config, name="config")
-        self.plugin_manager.add_hookspecs(Hooks)
-        self.plugin_manager.hook.add_config_validator(settings=self.settings)
-        self.settings.validators.validate()
+        self.pm.add_hookspecs(Hooks)
+        self.pm.hook.add_config_validator(settings=self.settings)
+        self._validate_settings()
+
         config_plugins = self.settings.default_plugins
 
         # the 'import' plugin maps to the 'moe_import' package
@@ -344,22 +367,22 @@ class Config:
             config_plugins[import_index] = "moe_import"
 
         if "cli" in config_plugins:
-            self.plugin_manager.register(importlib.import_module("moe.cli"), name="cli")
+            self.pm.register(importlib.import_module("moe.cli"), name="cli")
 
         # register plugin hookimpls for all enabled plugins
         self._register_internal_plugins(config_plugins)
 
         # register plugin hookimpls for all extra plugins
         for extra_plugin in self._extra_plugins:
-            self.plugin_manager.register(extra_plugin.plugin, extra_plugin.name)
+            self.pm.register(extra_plugin.plugin, extra_plugin.name)
 
         # register individual plugin sub-modules
-        self.plugin_manager.hook.plugin_registration(
-            config=self, plugin_manager=self.plugin_manager
-        )
+        self.pm.hook.plugin_registration(pm=self.pm)
 
         # register plugin hookspecs for all enabled plugins
-        self.plugin_manager.hook.add_hooks(plugin_manager=self.plugin_manager)
+        self.pm.hook.add_hooks(pm=self.pm)
+
+        log.debug(f"Registered plugins. [plugins={self.pm.list_name_plugin()}]")
 
     def _register_internal_plugins(self, enabled_plugins):
         """Registers all internal plugins in `enabled_plugins`."""
@@ -370,48 +393,4 @@ class Config:
 
             if plugin_path.stem in enabled_plugins:
                 plugin = importlib.import_module("moe.plugins." + plugin_name)
-                self.plugin_manager.register(plugin, plugin_name)
-
-
-def _edit_new_items(
-    session: sqlalchemy.orm.Session,
-    flush_context: sqlalchemy.orm.UOWTransaction,
-    instances: Optional[Any],
-    config: Config,
-):
-    """Runs the ``edit_new_items`` hook specification.
-
-    This uses the sqlalchemy ORM event ``before_flush`` in the background to determine
-    the time of execution and to provide any new or changed items to the hook
-    implementations.
-
-    Args:
-        session: Current db session.
-        flush_context: sqlalchemy obj which handles the details of the flush.
-        instances: List of objects passed to the ``flush()`` method.
-        config: Moe config.
-    """
-    config.plugin_manager.hook.edit_new_items(
-        config=config, items=session.new.union(session.dirty)
-    )
-
-
-def _process_new_items(
-    session: sqlalchemy.orm.Session,
-    flush_context: sqlalchemy.orm.UOWTransaction,
-    config: Config,
-):
-    """Runs the ``process_new_items`` hook specification.
-
-    This uses the sqlalchemy ORM event ``after_flush`` in the background to determine
-    the time of execution and to provide any new or changed items to the hook
-    implementations.
-
-    Args:
-        session: Current db session.
-        flush_context: sqlalchemy obj which handles the details of the flush.
-        config: Moe config.
-    """
-    config.plugin_manager.hook.process_new_items(
-        config=config, items=session.new.union(session.dirty)
-    )
+                self.pm.register(plugin, plugin_name)
