@@ -1,17 +1,17 @@
 """Import prompt."""
 
 import logging
-from typing import List
 
 import pluggy
 
 import moe
 import moe.cli
-from moe.config import Config
+from moe import config
 from moe.library.album import Album
-from moe.plugins import add as moe_add
+from moe.util.cli import PromptChoice, choice_prompt, fmt_item_changes
+from moe.util.core import get_matching_tracks
 
-log = logging.getLogger("moe.add")
+log = logging.getLogger("moe.cli.import")
 
 __all__ = ["AbortImport", "import_prompt"]
 
@@ -25,13 +25,12 @@ class Hooks:
 
     @staticmethod
     @moe.hookspec
-    def add_import_prompt_choice(prompt_choices: List[moe.cli.PromptChoice]):
+    def add_import_prompt_choice(prompt_choices: list[PromptChoice]):
         """Add a user input choice to the import prompt.
 
         ``func`` should return the album to be added to the library (or ``None`` if no
         album should be added) and will be supplied the following keyword arguments:
 
-            * ``config``: Moe config.
             * ``old_album``: Old album with no changes applied.
             * ``new_album``: New album consisting of all the new changes.
 
@@ -51,45 +50,47 @@ class Hooks:
 
 
 @moe.hookimpl
-def add_hooks(plugin_manager: pluggy.manager.PluginManager):
+def add_hooks(pm: pluggy.manager.PluginManager):
     """Registers `import` cli hookspecs to Moe."""
     from moe.plugins.moe_import.import_cli import Hooks
 
-    plugin_manager.add_hookspecs(Hooks)
+    pm.add_hookspecs(Hooks)
 
 
 @moe.hookimpl
-def process_candidates(config: Config, old_album: Album, candidates):
+def process_candidates(old_album: Album, candidates):
     """Use the import prompt to select and process the imported candidate albums."""
     if candidates:
+        chosen_candidate = candidates[0]
+        log.debug(
+            "Candidate album chosen for import prompt. "
+            f"[candidate={chosen_candidate!r}]"
+        )
         try:
-            import_prompt(config, old_album, candidates[0])
+            import_prompt(old_album, chosen_candidate)
         except AbortImport as err:
+            log.debug(err)
             raise SystemExit(0) from err
 
 
 @moe.hookimpl
-def add_import_prompt_choice(prompt_choices: List[moe.cli.PromptChoice]):
+def add_import_prompt_choice(prompt_choices: list[PromptChoice]):
     """Adds the ``apply`` and ``abort`` prompt choices to the user prompt."""
     prompt_choices.append(
-        moe.cli.PromptChoice(
-            title="Apply changes", shortcut_key="a", func=_apply_changes
-        )
+        PromptChoice(title="Apply changes", shortcut_key="a", func=_apply_changes)
     )
     prompt_choices.append(
-        moe.cli.PromptChoice(title="Abort", shortcut_key="x", func=_abort_changes)
+        PromptChoice(title="Abort", shortcut_key="x", func=_abort_changes)
     )
 
 
 def import_prompt(
-    config: Config,
     old_album: Album,
     new_album: Album,
 ):
     """Runs the interactive prompt for the given album changes.
 
     Args:
-        config: Moe config.
         old_album: Album to be added. Any changes will be applied to this album.
         new_album: New album with all metadata changes. Will be compared against
             ``old_album``.
@@ -97,25 +98,25 @@ def import_prompt(
     Raises:
         AbortImport: Import prompt was aborted by the user.
     """
-    if old_album == new_album:
-        return
+    log.debug("Running import prompt. [{old_album=!r}, {new_album=!r}]")
 
-    print(moe.cli.fmt_album_changes(old_album, new_album))
+    print(fmt_item_changes(old_album, new_album))
 
-    prompt_choices: List[moe.cli.PromptChoice] = []
-    config.plugin_manager.hook.add_import_prompt_choice(prompt_choices=prompt_choices)
+    prompt_choices: list[PromptChoice] = []
+    config.CONFIG.pm.hook.add_import_prompt_choice(prompt_choices=prompt_choices)
 
-    prompt_choice = moe.cli.choice_prompt(prompt_choices)
-    prompt_choice.func(config, old_album, new_album)
+    prompt_choice = choice_prompt(prompt_choices)
+    prompt_choice.func(old_album, new_album)
 
 
 def _apply_changes(
-    config: Config,
     old_album: Album,
     new_album: Album,
 ):
     """Applies the album changes."""
-    for old_track, new_track in moe_add.get_matching_tracks(old_album, new_album):
+    log.debug("Applying changes from import prompt.")
+
+    for old_track, new_track in get_matching_tracks(old_album, new_album):
         if not old_track and new_track:
             new_album.tracks.remove(new_track)  # missing track
         elif old_track and not new_track:
@@ -133,9 +134,8 @@ def _apply_changes(
 
 
 def _abort_changes(
-    config: Config,
     old_album: Album,
     new_album: Album,
 ):
     """Aborts the album changes."""
-    raise AbortImport()
+    raise AbortImport("Import prompt aborted; no changes made.")
