@@ -1,11 +1,11 @@
 """Core api for importing albums."""
 
-from typing import List
+import logging
 
 import pluggy
 
 import moe
-from moe.config import Config
+from moe import config
 from moe.library.album import Album
 from moe.library.extra import Extra
 from moe.library.lib_item import LibItem
@@ -13,13 +13,15 @@ from moe.library.track import Track
 
 __all__ = ["import_album"]
 
+log = logging.getLogger("moe.import")
+
 
 class Hooks:
     """Import core plugin hook specifications."""
 
     @staticmethod
     @moe.hookspec
-    def import_candidates(config: Config, album: Album) -> Album:  # type: ignore
+    def import_candidates(album: Album) -> Album:  # type: ignore
         """Imports candidate albums from implemented sources based on the given album.
 
         This hook should be used to import metadata from an external source and return
@@ -30,7 +32,6 @@ class Hooks:
             This hook runs within the :meth:`pre_add` hook.
 
         Args:
-            config: Moe config.
             album: Album being added to the library.
 
         Returns:
@@ -39,7 +40,7 @@ class Hooks:
 
     @staticmethod
     @moe.hookspec
-    def process_candidates(config: Config, old_album: Album, candidates: List[Album]):
+    def process_candidates(old_album: Album, candidates: list[Album]):
         """Process the imported candidate albums.
 
         If you wish to save and apply any candidate album metadata, it should be applied
@@ -49,22 +50,21 @@ class Hooks:
         resolved.
 
         Args:
-            config: Moe config.
             old_album: Album being added to the library.
             candidates: New candidate albums with imported metadata.
         """
 
 
 @moe.hookimpl
-def add_hooks(plugin_manager: pluggy.manager.PluginManager):
+def add_hooks(pm: pluggy.manager.PluginManager):
     """Registers `import` core hookspecs to Moe."""
     from moe.plugins.moe_import.import_core import Hooks
 
-    plugin_manager.add_hookspecs(Hooks)
+    pm.add_hookspecs(Hooks)
 
 
 @moe.hookimpl
-def pre_add(config: Config, item: LibItem):
+def pre_add(item: LibItem):
     """Fixes album metadata via external sources prior to it being added to the lib."""
     if isinstance(item, Album):
         album = item
@@ -73,16 +73,17 @@ def pre_add(config: Config, item: LibItem):
     else:
         raise NotImplementedError
 
-    import_album(config, album)
+    import_album(album)
 
 
-def import_album(config: Config, album: Album):
+def import_album(album: Album):
     """Imports album metadata for an album."""
-    candidates = config.plugin_manager.hook.import_candidates(
-        config=config, album=album
-    )
-    config.plugin_manager.hook.process_candidates(
-        config=config,
+    log.debug(f"Importing album metadata. [{album=!r}]")
+
+    candidates = config.CONFIG.pm.hook.import_candidates(album=album)
+    config.CONFIG.pm.hook.process_candidates(
         old_album=album,
         candidates=candidates,
     )
+
+    log.debug(f"Imported album metadata. [{album=!r}]")
