@@ -11,7 +11,7 @@ import dynaconf
 from unidecode import unidecode
 
 import moe
-from moe.config import Config
+from moe import config
 from moe.library.album import Album
 from moe.library.extra import Extra
 from moe.library.lib_item import LibItem
@@ -26,113 +26,71 @@ log = logging.getLogger("moe.move")
 def add_config_validator(settings: dynaconf.base.LazySettings):
     """Validate move plugin configuration settings."""
     default_album_path = "{album.artist}/{album.title} ({album.year})"
-    default_extra_path = "{extra.filename}"
+    default_extra_path = "{extra.path.name}"
     default_track_path = (
         "{f'Disc {track.disc:02}' if track.disc_total > 1 else ''}/"
         "{track.track_num:02} - {track.title}{track.path.suffix}"
     )
 
-    settings.validators.register(
-        dynaconf.Validator("MOVE.ASCIIFY_PATHS", default=False)
-    )
-    settings.validators.register(
-        dynaconf.Validator("MOVE.ALBUM_PATH", default=default_album_path)
-    )
-    settings.validators.register(
-        dynaconf.Validator("MOVE.EXTRA_PATH", default=default_extra_path)
-    )
-    settings.validators.register(
-        dynaconf.Validator("MOVE.TRACK_PATH", default=default_track_path)
-    )
+    validators = [
+        dynaconf.Validator("MOVE.ASCIIFY_PATHS", default=False),
+        dynaconf.Validator("MOVE.ALBUM_PATH", default=default_album_path),
+        dynaconf.Validator("MOVE.EXTRA_PATH", default=default_extra_path),
+        dynaconf.Validator("MOVE.TRACK_PATH", default=default_track_path),
+    ]
+    settings.validators.register(*validators)
 
 
-@moe.hookimpl
-def post_add(config: Config, item: LibItem):
+@moe.hookimpl(trylast=True)
+def edit_new_items(items: list[LibItem]):
     """Copies and formats the path of an item after it has been added to the library."""
-    # copy the whole album in case album attributes have changed
-    if isinstance(item, Album):
-        album = item
-    elif isinstance(item, (Extra, Track)):
-        album = item.album_obj
-    else:
-        raise NotImplementedError
-
-    copy_item(config, album)
+    for item in items:
+        copy_item(item)
 
 
 ########################################################################################
 # Format paths
 ########################################################################################
-def fmt_item_path(config: Config, item: LibItem) -> Path:
-    """Returns a formatted item path according to the user configuration.
+def fmt_item_path(item: LibItem) -> Path:
+    """Returns a formatted item path according to the user configuration."""
+    log.debug(f"Formatting item path. [path={item.path}]")
 
-    Args:
-        config: Moe config.
-        item: Library item used to format the directory.
-
-    Returns:
-        Formatted item path under the config ``library_path``.
-
-    Raises:
-        NotImplementedError: Unknown item.
-    """
     if isinstance(item, Album):
-        new_path = _fmt_album_path(config, item)
+        new_path = _fmt_album_path(item)
     elif isinstance(item, Extra):
-        new_path = _fmt_extra_path(config, item)
+        new_path = _fmt_extra_path(item)
     elif isinstance(item, Track):
-        new_path = _fmt_track_path(config, item)
+        new_path = _fmt_track_path(item)
     else:
         raise NotImplementedError
 
-    if config.settings.move.asciify_paths:
+    if config.CONFIG.settings.move.asciify_paths:
         new_path = Path(unidecode(str(new_path)))
 
+    log.debug(f"Formatted item path. [path={new_path}]")
     return new_path
 
 
-def _fmt_album_path(config: Config, album: Album) -> Path:
-    """Returns a formatted album directory according to the user configuration.
-
-    An album directory should contain, at a minimum, the album artist, title, and year
-    to ensure uniqueness.
-
-    Args:
-        config: Moe config.
-        album: Album used to format the directory.
-
-    Returns:
-        Formatted album directory under the config ``library_path``.
-    """
-    library_path = Path(config.settings.library_path).expanduser()
-    album_path = _eval_path_template(config.settings.move.album_path, album)
+def _fmt_album_path(album: Album) -> Path:
+    """Returns a formatted album directory according to the user configuration."""
+    library_path = Path(config.CONFIG.settings.library_path).expanduser()
+    album_path = _eval_path_template(config.CONFIG.settings.move.album_path, album)
 
     return library_path / album_path
 
 
-def _fmt_extra_path(config: Config, extra: Extra) -> Path:
+def _fmt_extra_path(extra: Extra) -> Path:
     """Returns a formatted extra path according to the user configuration."""
-    album_path = _fmt_album_path(config, extra.album_obj)
-    extra_path = _eval_path_template(config.settings.move.extra_path, extra)
+    album_path = _fmt_album_path(extra.album_obj)
+    extra_path = _eval_path_template(config.CONFIG.settings.move.extra_path, extra)
 
     return album_path / extra_path
 
 
-def _fmt_track_path(config: Config, track: Track) -> Path:
-    """Returns a formatted track path according to the user configuration.
-
-    The track path should contain, at a minimum, the track number and
-    disc (if more than one) to ensure uniqueness.
-
-    Args:
-        track: Track used to format the path.
-        config: Moe config.
-
-    Returns:
-        Formatted track path under its album path.
-    """
-    album_path = _fmt_album_path(config, track.album_obj)
-    track_path = _eval_path_template(config.settings.move.track_path, track)
+def _fmt_track_path(track: Track) -> Path:
+    """Returns a formatted track path according to the user configuration."""
+    album_path = _fmt_album_path(track.album_obj)
+    track_path = _eval_path_template(config.CONFIG.settings.move.track_path, track)
 
     return album_path / track_path
 
@@ -147,9 +105,6 @@ def _eval_path_template(template, lib_item) -> str:
 
     Returns:
         Evaluated path.
-
-    Raises:
-        NotImplementedError: You discovered a new library item!
     """
     template_parts = template.split("/")
     sanitized_parts = []
@@ -170,7 +125,7 @@ def _lazy_fstr_item(template: str, lib_item: LibItem) -> str:
             All library items should have their own template and refer to variables as:
                 Album: album (e.g. {album.title}, {album.artist})
                 Track: track (e.g. {track.title}, {track.artist})
-                Extra: extra (e.g. {extra.filename}
+                Extra: extra (e.g. {extra.path.name}
         lib_item: Library item referenced in the template.
 
 
@@ -227,102 +182,88 @@ def _sanitize_path_part(path_part: str) -> str:
 ########################################################################################
 # Copy
 ########################################################################################
-def copy_item(config: Config, item: LibItem):
+def copy_item(item: LibItem):
     """Copies an item to a destination as determined by the user configuration.
 
     Overwrites any existing files. Will create the destination if it does not already
     exist.
-
-    Args:
-        item: Library item to copy.
-        config: Moe config.
     """
     if isinstance(item, Album):
-        _copy_album(config, item)
+        _copy_album(item)
     elif isinstance(item, (Extra, Track)):
-        _copy_file_item(config, item)
+        _copy_file_item(item)
 
 
-def _copy_album(config: Config, album: Album):
-    """Copies an album to a destination as determined by the user configuration.
+def _copy_album(album: Album):
+    """Copies an album to a destination as determined by the user configuration."""
+    dest = fmt_item_path(album)
 
-    Copying an album will also copy all of it's tracks and extras.
+    log.debug(f"Copying album. [{dest=}, {album=!r}]")
 
-    Args:
-        album: Album to copy
-        config: Moe config.
-    """
-    dest = fmt_item_path(config, album)
-
-    log.info(f"Copying album from '{album.path}' to '{dest}'.")
     if album.path != dest:
         dest.mkdir(parents=True, exist_ok=True)
         album.path = dest
 
     for track in album.tracks:
-        _copy_file_item(config, track)
+        _copy_file_item(track)
 
     for extra in album.extras:
-        _copy_file_item(config, extra)
+        _copy_file_item(extra)
+
+    log.info(f"Album copied. [{dest=}, {album=!r}]")
 
 
-def _copy_file_item(config: Config, item: Union[Extra, Track]):
+def _copy_file_item(item: Union[Extra, Track]):
     """Copies an extra or track to a destination as determined by the user config."""
-    dest = fmt_item_path(config, item)
+    dest = fmt_item_path(item)
+    log.debug(f"Copying item. [{dest=}, {item=!r}]")
+
     if dest == item.path:
         return
 
-    log.info(f"Copying {type(item).__name__.lower()} from '{item.path}' to '{dest}'.")
     dest.parent.mkdir(parents=True, exist_ok=True)
     shutil.copyfile(item.path, dest)
 
     item.path = dest
 
+    log.info(f"Copied item. [{dest=}, {item=!r}]")
+
 
 ########################################################################################
 # Move
 ########################################################################################
-def move_item(config: Config, item: LibItem):
+def move_item(item: LibItem):
     """Moves an item to a destination as determined by the user configuration.
 
     Overwrites any existing files. Will create the destination if it does not already
     exist.
-
-    Args:
-        item: Library item to move.
-        config: Moe config.
     """
     if isinstance(item, Album):
-        _move_album(config, item)
+        _move_album(item)
     elif isinstance(item, (Extra, Track)):
-        _move_file_item(config, item)
+        _move_file_item(item)
 
 
-def _move_album(config: Config, album: Album):
+def _move_album(album: Album):
     """Moves an album to a given destination.
 
-    - Overwrites any existing files.
-    - Creates the destination if it does not already exist.
-    - Tracks and extras are also moved.
-    - Empty leftover directories will be removed.
-
-    Args:
-        album: Album to move.
-        config: Moe config.
+    Note:
+        Empty leftover directories will be removed.
     """
-    dest = fmt_item_path(config, album)
+    dest = fmt_item_path(album)
     old_album_dir = album.path
 
-    log.info(f"Moving album from '{album.path}' to '{dest}'.")
+    log.debug(f"Moving album. [{dest=}, {album=!r}]")
+
     if album.path != dest:
         dest.mkdir(parents=True, exist_ok=True)
         album.path = dest
 
     for track in album.tracks:
-        _move_file_item(config, track)
+        _move_file_item(track)
 
     for extra in album.extras:
-        _move_file_item(config, extra)
+        _move_file_item(extra)
 
     # remove any empty leftover directories
     for old_child in old_album_dir.rglob("*"):
@@ -334,15 +275,20 @@ def _move_album(config: Config, album: Album):
         with suppress(OSError):
             old_parent.rmdir()
 
+    log.info(f"Moved album. [{dest=}, {album=!r}]")
 
-def _move_file_item(config: Config, item: Union[Extra, Track]):
+
+def _move_file_item(item: Union[Extra, Track]):
     """Moves an extra or track to a destination as determined by the user config."""
-    dest = fmt_item_path(config, item)
+    dest = fmt_item_path(item)
     if dest == item.path:
         return
 
-    log.info(f"Moving {type(item).__name__.lower()} from '{item.path}' to '{dest}'.")
+    log.debug(f"Moving item. [{dest=}, {item=!r}]")
+
     dest.parent.mkdir(parents=True, exist_ok=True)
     item.path.replace(dest)
 
     item.path = dest
+
+    log.info(f"Moved item. [{dest=}, {item=!r}]")
