@@ -3,7 +3,7 @@
 import pluggy
 
 import moe
-from moe.config import Config
+from moe import config
 
 from . import rm_cli, rm_core
 from .rm_core import *
@@ -13,8 +13,8 @@ __all__.extend(rm_core.__all__)
 
 
 @moe.hookimpl
-def plugin_registration(config: Config):
+def plugin_registration():
     """Only register the cli sub-plugin if the cli is enabled."""
-    config.plugin_manager.register(rm_core, "remove_core")
-    if config.plugin_manager.has_plugin("cli"):
-        config.plugin_manager.register(rm_cli, "remove_cli")
+    config.CONFIG.pm.register(rm_core, "remove_core")
+    if config.CONFIG.pm.has_plugin("cli"):
+        config.CONFIG.pm.register(rm_cli, "remove_cli")
